-- some vars
local myappwidth, myappheight = application:getContentWidth(), application:getContentHeight()
local matrix = Matrix.new()

-- react physics 3D
r3d = require "reactphysics3d"
local world = r3d.World.new(0, -9.8, 0) -- 0, -9.8, 0
world.staticbodies = {} -- staticbodies list
world.cubes = {}

-- tiled
local level = loadfile("tiled/r3d_map01.lua")()
local tilew, tilel = level.tilewidth, level.tileheight
local tiledwidth, tiledlength = level.width * tilew, level.height * tilel

-- a fullscreen 3D viewport
local view = D3.View.new(myappwidth, myappheight, 45, 1, tiledlength * tiledlength / 2) -- function D3View:init(width, height, fov, near, far)

-- a sky
local sky = Pixel.new(0xffffff, 1, myappwidth * 2, myappheight * 2)
sky:setAnchorPoint(0.5, 0.5)
sky:setPosition(myappwidth / 2, myappheight / 2)
sky:setColor(0x00FFFF, 1, 0x00000FF, 1, 90) -- 0x00040FF -- blue gradient background
stage:addChild(sky)

-- some ground
local ground = PLANE_3D(world, {
		sizex=tiledwidth/2, sizez=tiledlength/2, texpath="3d_objs/texs/groundx01.png", texw=64*64, texh=64*64
	})

-- tiled map
local layers = level.layers
for i = 1, #layers do
	local layer = layers[i]
	local layerType = layer.type
	if layerType == "imagelayer" then
		-- process image layer here
	elseif layerType == "objectgroup" then
		local objects = layer.objects
		local index = 1
		for i = 1, #objects do
			local object = objects[i]
			if object.name == "wall" then
				cube_factory( world, {
					sizex = object.width / 2,
					sizey = 32,
					sizez = object.height / 2,
					posx = object.x + object.width / 2 - tiledlength / 2,
					posy = 32,
					posz = -object.y - object.height / 2 + tiledlength / 2,
					roty = object.rotation,
					texpath = "3d_objs/texs/box.png", texw = 32 * 64, texh = 32 * 64 }
				)
			elseif object.name == "start" then
			elseif object.name == "exit" then
			else
				print("unknown object type")
			end
		end
	else
		print("unknown layer type")
	end
end

-- an object
--local patrick = loadObj("3d_objs/Patrick Star", "patrick.obj", "3d_objs/Patrick Star/Tex_0086_2.png")
--local patrick = loadObj("3d_objs/Patrick Star", "patrick.obj")

-- the scene
local scene = view:getScene() -- a 3D View sprite
stage:addChild(view)

-- add the objects to the scene
for p = 1, #world.staticbodies do scene:addChild(world.staticbodies[p]) end
for k, v in pairs(world.cubes) do scene:addChild(k) end

-- position the camera
view:lookAt(0*64,	16*64,	-4*64,
			-1.5*64,	0.2*64,	1*64
)

-- the game loop
stage:addEventListener(Event.ENTER_FRAME, function(e)
	-- action!
	world:step(e.deltaTime)
	for k, v in pairs(world.cubes) do
		k:setMatrix(v.xbody:getTransform())
	end
end)
